#property copyright "Public domain software by Ralph Ronnquist"
#property link      ""

// Tells the broker's current time zone
extern int timezone=0;
extern int magic = 96;

extern int SAMPLE_HOUR = 7;
extern int EXPIRES = 6;
extern int CLOSE_HOUR = 6;
extern double lots = 1.0;

int state = 0;

int init()
{
    return( 0 );
}

int deinit()
{
    return( 0 );
}

int start()
{
    // Track time of first tick after that the EA is dropped onto the chart.
    static datetime first_tick = 0;
    if ( first_tick == 0 ) first_tick = TimeCurrent();

    int i;
    double price;
    bool some_open = false;

    int h = TimeHour( TimeCurrent() ) - timezone;
    if ( h < 0 ) h += 24;

    // Detrmine opening edge, takeprofit and stoploss by using the simple moving
    // average of the five 24-hour periods before the most recent sample hour.
    int OP = sampleRange() / 2;
    int TP = OP * 1.5;
    int SL = OP * 0.5;
    
    // Review open trades in order to apply the closing condition. It also determines
    // whether there are any open trades, in which case no more trades should be opened.
    for ( i = OrdersTotal() - 1; i >=0; i-- ) {
        if ( ! OrderSelect( i, SELECT_BY_POS, MODE_TRADES ) )
            continue;
        if ( magic != 0 && OrderMagicNumber() != magic )
            continue;
        if ( OrderSymbol() != Symbol() )
            continue;
        if ( OrderType() == OP_BUY ) {
            if ( h != CLOSE_HOUR ) {
                if ( Bid < OrderOpenPrice() + TP * Point &&
                     Bid > OrderOpenPrice() - SL * Point ) {
                    some_open = true;
                    continue;
                }
            }
            price = Bid;
        } else if ( OrderType() == OP_SELL ) {
            if ( h != CLOSE_HOUR ) {
                if ( Ask > OrderOpenPrice() - TP * Point &&
                     Ask < OrderOpenPrice() + SL * Point ) {
                    some_open = true;
                    continue;
                }
            }
            price = Ask;
        }
        if ( ! OrderClose( OrderTicket(), OrderLots(), price, Yellow ) )
            some_open = true;
    }
    if ( some_open )
        return( 0 );

    // Determine the date stamp for the sample
    // gmt_day is the current date at GMT time
    // sample_time is the time stamp the broker has for the selected GMT sample hour
    string gmt_day = TimeToStr( TimeCurrent() - timezone * 3600, TIME_DATE );
    datetime sample_time = StrToTime( gmt_day ) + ( SAMPLE_HOUR + timezone ) * 3600 ;
    
    datetime last_sample = sample_time;
    if ( last_sample > TimeCurrent() )
        last_sample -= 86400;

    // Review account history for a trade after the sample time, so as to avoid
    // a second one.
    for ( i = OrdersHistoryTotal() - 1; i >=0; i-- ) {
        if ( ! OrderSelect( i, SELECT_BY_POS, MODE_HISTORY ) )
            continue;
        if ( magic != 0 && OrderMagicNumber() != magic )
            continue;
        if ( OrderSymbol() != Symbol() )
            continue;
        if ( OrderOpenTime() >= last_sample ) {
            return( 0 );
        }
    }

    // Avoid trading at today's sample time if  it is before the first EA tick time
    if ( sample_time < first_tick ) return( 0 );

    // Limit the enabling period to be EXPIRES hours after sample SAMPLE_HOUR each day.
    if ( TimeCurrent() < sample_time ) return( 0 ); // Too early
    if ( TimeCurrent() >= sample_time + EXPIRES * 3600 ) return( 0 ); // Too late

    double sample_price =
        iOpen( Symbol(), PERIOD_H1, iBarShift( Symbol(), PERIOD_H1, sample_time, true ) );

    // Check for opening condition
    int type = -1;
    if ( Ask >= sample_price + OP * Point ) {
        type = OP_BUY;
        price = Ask;
    } else if ( Bid <= sample_price - OP * Point ) {
        type = OP_SELL;
        price = Bid;
    } else {
        return( 0 );
    }
    OrderSend( Symbol(), type, lots, price, 0, 0, 0, "", magic, 0, CLR_NONE );
    return(0);
}

//
// Determines the simple moving average of the last five 24-hour periods
// prior to the given SAMPLE_HOUR.
//
int sampleRange()
{
    int bars = iBars( Symbol(), PERIOD_H1 );
    for ( int bar = 0; bar < bars; bar++ ) {
        int h = TimeHour( iTime( Symbol(), PERIOD_H1, bar ) ) - timezone;
        if ( h < 0 )
            h += 24;
        if ( h == SAMPLE_HOUR )
            break;
    }
    if ( bar >= bars )
        return( 0 );
    return( rangeAverage( bar ) );
}

// Computes the moving average of the last five 24-hour periods prior to
// the given H1 bar.
int rangeAverage(int bar)
{
    int sum = 0;
    for ( int i = 0; i < 5; i++ ) {
        sum += range( bar );
    }
    return( sum / 5 );
}        

// Computes the price range of the 24-hour period prior to the given H1 bar,
// and it also moves that index to the beginning of that 24-hour period.
// Note: doesn't handle missing bars very well.
int range(int &bar)
{
    int h = TimeHour( iTime( Symbol(), PERIOD_H1, bar ) );
    double hi = 0;
    double lo = 10000;
    int state = 1;
    for ( bar = bar + 1; bar < Bars; bar++ ) {
        hi = MathMax( hi, iHigh( Symbol(), PERIOD_H1, bar ) );
        lo = MathMin( lo, iLow( Symbol(), PERIOD_H1, bar  ) );
        if ( h == TimeHour( iTime( Symbol(), PERIOD_H1, bar ) ) )
            break;
    }
    return( ( hi - lo ) / Point );
}

